# OBJ - timestamped JSON objects
#
# obj/cli.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-09-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" command line interface. """

__version__ = 1

## imports

from obj.base import Obj
from obj.fleet import Fleet
from obj.handler import Handler
from obj.shell import cfg

import os
import sys
import threading

## instances

fleet = Fleet()

## classes

class Cfg(Obj):

    pass

class CLI(Handler):

    def __init__(self, threaded=False, verbose=True):
        super().__init__()
        self._ready = threading.Event()
        self._sock = sys.stdout
        self.threaded = threaded
        self.verbose = verbose

    def announce(self, txt):
        self.raw(txt)

    def cmd(self, txt):
        event = super().event(txt, "root@shell", options=cfg.options)
        self.dispatch(event)
        return event

    def event(self, txt, options=""):
        e = super().event(txt, "root@shell", options=options or cfg.options)
        return e

    def raw(self, txt):
        if not self.verbose:
            return
        self._sock.write(str(txt))
        self._sock.write("\n")
        self._sock.flush()

    def ready(self):
        self._ready.set()

    def say(self, channel, txt):
        self.raw(txt)

    def wait(self):
        self._ready.wait()

class Console(CLI):

    def __init__(self, prompted=True):
        super().__init__()
        self.prompted = prompted
        
    def dispatch(self, event):
        super().dispatch(event)
        self.prompt()

    def get_event(self):
        try:
            return self.event(input())
        except EOFError:
            return None            

    def prompt(self):
        if self.prompted:
            self._sock.write("> ")
            self._sock.flush()

