# OBJ - timestamped JSON objects
#
# obj/cmds.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-09-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    basic commands.
    
"""

## imports

from obj.base import Obj, starttime, __version__, __txt__
from obj.db import Db
from obj.fleet import Fleet
from obj.shell import set_completer
from obj.thr import Launcher
from obj.users import Users

import obj

import os
import json
import time
import _thread

## defines

psformat = "%-8s %-60s"

## instances

db = Db()
bots = Fleet()
launcher = Launcher()
users = Users()

## classes

class Log(Obj):

    pass

class Todo(Obj):

    pass

## commands

def cfg(event):
    """ cfg <type>. """
    if not event.args:
        event.reply("cfg <name>")
        return
    name = "obj.%s.Cfg" % event.args[0]
    try:
        cfg = Obj().last(name)
        event.reply(cfg)
        return
    except FileNotFoundError:
        event.reply("%s config not found" % event.args[0])

def cmds(event):
    """ show list of commands. """
    bot = bots.get_bot(event.orig)
    event.reply(",".join(sorted(bot.handlers.keys())))

def deleted(event):
    """ deleted <prefix> [match] - show objects with the deleted flag set. """
    if not event.args:
        event.reply("deleted what?")
        return
    type = event.args[0].lower()
    types = []
    for x in os.listdir(obj.base.workdir):
        if type in x.lower():
            types.append(x)
    if not types:
        types.append(type)
    for type in types:
        for fn, o in db.find(type, *event.dkeys, skip=False, **event.want):
            if "_deleted" in o and not o._deleted:
                continue
            event.display(o)

def ed(event):
    """ edit objects. """
    db = Db()
    try:
        fn, o = db.last(event.args[0])
    except IndexError:
        event.reply("ed <type>")
        return
    except IsADirectoryError as ex:
        event.reply("%s %s" % (event.txt, ex.strerror.lower()))
        return
    if not o:
        event.reply("no match found for %s" % " ".join(event.args))
        return
    if len(event.args) == 1:
        event.reply(o)
        return 
    key = event.args[1]
    if key not in o:
        event.reply("no %s attribute available." % key)
        return
    if len(event.args) == 2:
        event.reply('%s=%s' % (key, o[key]))
        return
    if len(event.args) == 3:
        val = event.args[2]
    elif len(event.args) >= 3:
        val = event.args[2:]
    try:
        val = int(val)
    except:
        try:
            val = float(val)
        except:
            pass
    t = type(o[key])
    tt = type(val)
    if t != tt:
        event.reply("%s != %s" % (tt, t))
        return
    if t == str:
        o[key] = val
    elif t == list:
        o[key] = val
    elif val in ["True", "true"]:
        o[key] = True
    elif val in ["False", "false"]:
        o[key] = False
    else:
        try:
            val = '"%s"' % val
            o[key] = json.loads(val)
        except json.decoder.JSONDecodeError:
            event.reply("%s is not JSON" % val)
        except (SyntaxError, ValueError, json.decode.JSONDecodeError) as ex:
            event.reply("%s: %s" % (str(ex), val))
        return
    o.save(fn)
    event.reply('ok %s="%s"' % (key, o[key]))

def exit(event):
    """ exit the program. """
    _thread.interrupt_main()

def fleet(event):
    """ show bots in fleet. """
    try:
        nr = int(event.args[0])
    except (IndexError, ValueError) as ex:
        event.reply([type(b) for b in bots.bots])
        return
    try:
        event.reply(str(bots.bots[nr]))
    except IndexError:
        event.reply("less than %s bots available." % nr)


def find(event):
    """ find <prefix> [match] - find objects with prefix and matching fields. """
    if len(event.args) < 1:
        event.reply("find <type> <match>")
        return
    type = event.args[0].lower()
    types = []
    fns = [(x.split(".")[-1].lower(), x) for x in os.listdir(obj.base.workdir) if type in x.split(".")[-1].lower()]
    for type, fn in fns:
        if type in fn.lower():
            types.append(fn)
    if not types:
        types.append(type)
    nr = 0
    for type in types:
        for fn, o in db.find(type, *event.dkeys, **event.want):
            if "t" in event.options:
                days = elapsed(time.time() - fn_time(fn))
            else:
                days = 0
            if days:
                event.display(o, txt=str(nr), post="- %s" % days)
            else:
                event.display(o, txt=str(nr))
            nr += 1

def kill(event):
    """ kill a task. """
    if not event.args:
        event.reply("kill <match>")
        return
    n = event.args[0]
    launcher.kill(n)
    event.reply("ok %s" % n)

def last(event):
    """ last <prefix> [match] - show last saved object. """
    if not event.args:
        event.reply("last what?")
        return
    type = event.args[0].lower()
    types = []
    for x in os.listdir(obj.base.workdir):
        if type in x.lower():
            types.append(x)
    if not types:
        types.append(type)
    for type in types:
        fn, o = db.last(type, *event.dkeys)
    event.reply(o)

def load(event):
    """ load modules. """
    bot = bots.get_bot(event.orig)
    if not event.args:
        event.reply("load obj.[%s]" % ",".join([x.split(".")[-1] for x in sorted(bot.list("obj"))]))
        return
    name = event.args[0]
    try:
        bot.walk(name, True)
    except ModuleNotFoundError as ex:
        event.reply(ex.msg.lower()) 
    set_completer(sorted(bot.handlers.keys()))
    event.reply("%s loaded" % name)

def log(event):
    """ log some text. """
    l = Log()
    l.txt = event.rest
    l.save()
    event.ok()

def ls(event):
    """ show possible prefixes (first argument of command). """
    if not event.args:
        event.reply(",".join(sorted(set([x.split(".")[-1].lower() for x in os.listdir(obj.base.workdir)]))))
    else:
        m = event.args[0].lower()
        event.reply(",".join(set([x.split(".")[-1].lower() for x in os.listdir(obj.base.workdir) if m in x.lower()])))

def meet(event):
    """ add a new user. """
    try:
        origin, *perms = event.args
    except ValueError:
        return
    origin = users.userhosts.get(origin, origin)
    if users.meet(origin, perms):
        event.reply("%s added." % origin)

def perm(event):
    """ set permission of a user. """
    try:
        origin, *perms = event.args
    except ValueError:
        return
    users.users.last()
    origin = users.userhosts.get(origin, origin)
    for perm in perms:
        u = users.perm(origin, perm)
        if u:
            event.reply("ok %s" % origin)

def ps(event):
    """ show running threads. """
    result = []
    for thr in sorted(launcher.running(), key=lambda x: x.getName()):
        o = Obj()
        o.update(vars(thr))
        if o.get("sleep", None):
            up = o.sleep - int(time.time() - o._time.latest)
        else:
            up = int(time.time() - starttime)
        thrname = thr.getName()
        result.append((up, thrname, o))
    nr = 0
    for up, thrname, o in sorted(result, key=lambda x: x[0]):
        res = "%s  %s" % (nr, psformat % (elapsed(up), thrname[:30]))
        event.reply(res.rstrip())
        nr += 1

def reboot(event):
    """ reboot the program. """
    bots.announce("reboot %s" % time.ctime(time.time()))
    bots.save()
    time.sleep(2.0)
    reset()
    os.execl(sys.argv[0], *sys.argv)

def rm(event):
    """ rm <prefix> [match] - set deleted flag. """
    if not event.args:
        event.reply("remove what?")
        return
    type = event.args[0].lower()
    types = []
    for x in os.listdir(obj.base.workdir):
        if type in x.lower():
            types.append(x)
    if not types:
        types.append(type)
    nr = 0
    for type in types:
        for fn, o in db.find(type, *event.dkeys, skip=False, **event.want):
            if "_deleted" not in dir(o) and not o._deleted:
                o._deleted = True
                o.save(fn)
                nr += 1
    event.reply("ok %s" % nr)

def rmperm(event):
    """ remove permission of a user. """
    try:
        origin, perm = event.args
    except:
        return
    origin = users.userhosts.get(origin, origin)
    result = users.delete(origin, perm)
    if not result:
        return
    event.reply("ok %s %s" % (origin, perm))

def test(event):
    """ echo a reply. """
    event.reply("yoo %s" % event.origin)

def todo(event):
    """ save a todo item. """
    t = Todo()
    t.txt = event.rest
    t.save()
    event.ok()

def unload(event):
    """ unload modules. """
    bot = bots.get_bot(event.orig)
    if not event.args:
        event.reply("unload obj.[%s]" % ",".join([x.split(".")[-1] for x in sorted(bot.table.keys())]))
        return
    name = event.args[0]
    if name == "obj.cmds":
        event.reply("can't unload obj.cmds")
        return
    try:
        bot.unload(name)
    except KeyError:
        event.reply("%s is not unloadable." % name)
        return
    set_completer(sorted(bot.keys()))
    event.reply("%s unloaded" % name)

def uptime(event):
    """ show uptime. """
    event.reply("uptime is %s" % elapsed(time.time() - starttime))

def user(event):
    """ show user. """
    if not event.args:
        return
    origin = event.args[0]
    origin = users.userhosts.get(origin, origin)
    u = users.users.get(origin, None)
    event.reply(u)

def version(event):
    """ show version. """
    event.reply("OBJ %s - %s" % (__version__, __txt__))

## helpers

def elapsed(seconds, short=True):
    """ show elapsed time. """
    txt = ""
    sub = str(seconds).split(".")[-1]
    nsec = float(seconds)
    year = 365*24*60*60
    week = 7*24*60*60
    day = 24*60*60
    hour = 60*60
    minute = 60
    years = int(nsec/year)
    nsec -= years*year
    weeks = int(nsec/week)
    nsec -= weeks*week
    days = int(nsec/day)
    nsec -= days*day
    hours = int(nsec/hour)
    nsec -= hours*hour
    minutes = int(nsec/minute)
    sec = nsec - minutes*minute
    if years:
        txt += "%sy" % years
    if weeks:
        days += weeks * 7
    if days:
        txt += "%sd" % days
    if years and short and txt:
        return txt
    if hours:
        txt += "%sh" % hours
    if days and short and txt:
        return txt
    if minutes:
        txt += "%sm" % minutes
    if hours and short and txt:
        return txt
    if sec == 0:
        txt += "0s"
    elif sec < 1 or not short:
        txt += "%.3fs" % sec
    else:
        txt += "%ss" % int(sec)
    txt = txt.strip()
    return txt
