# OBJ - timestamped JSON objects
#
# obj/db.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-09-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

"""
    cursor on the datastore, works with timestamped filenames.
    
"""

__version__ = 1

## imports

from obj.base import Obj, Register

import functools
import logging
import obj.base
import os

## classes

class Db(Obj):

    def filenames(self, path):
        res = []
        p = os.path.join(obj.base.workdir, path.strip())
        for rootdir, dirs, files in os.walk(p, topdown=True):
            if p not in rootdir:
                continue
            for fn in files:
                yield os.path.join(rootdir, fn).split(obj.base.workdir)[-1][1:]

    def deleted(self, path):
        for fn in sorted(self.filenames(path)):
            o = Obj().load(fn)
            if "_deleted" in dir(o) and o._deleted:
                yield fn, o
        
    def find(self, path, match="", skip=True, *args, **kwargs):
        if not path:
            return []
        for fn in sorted(self.filenames(path)):
            o = Obj(_default="").load(fn)
            if skip and "_deleted" in dir(o) and o._deleted:
                continue
            if kwargs:
                go = False
                for k, v in kwargs.items():
                    vv = o.get(k, None)
                    if vv and v in str(vv):
                        go = True
                if not go:
                    continue
            if match and o.match(match):
                yield fn, o
            else:
                yield fn, o

    def last(self, path, match="", *args):
        fns = sorted(self.filenames(path))
        for fn in fns[::-1]:
            o = Obj().load(fn)
            if o.match(match):
                return fn, o
        return None, None
