# OBJ - timestamped JSON objects
#
# obj/loader.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-09-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" a loader is an object that can load modules. """

__version__ = 1

## imports

from obj.base import Obj 

import importlib
import logging
import obj
import os
import pkgutil
import sys
import types

## classes

class Loader(Obj):

    def __init__(self):
        super().__init__()
        self.table = Obj()
    
    def direct(self, name, package=None):
        logging.info("direct %s" % name)
        return importlib.import_module(name, package)

    def list(self, name):
        res = []
        for name in pkgutil.walk_packages(obj.__path__, obj.__name__+"."):
            res.append(name[1])
        return res
                        
    def load_mod(self, modname, mod, service=False):
        self.table[modname] = mod
        if service:
            func = getattr(mod, "start", None)
            if func:
                logging.warn("service %s" % modname)
                self.launch(func)
        return mod

    def unload(self, modname):
        del self.table[modname]

    def walk(self, name, service=False):
        for n in pkgutil.walk_packages(obj.__path__, obj.__name__+'.'):
            if name in n[1]:
                try:
                    mod = self.direct(n[1])
                    self.load_mod(n[1], mod, service)
                except ModuleNotFoundError as ex:
                    logging.error(str(ex))
