# OBJ - timestamped JSON objects
#
# obj/thr.py
#
# Copyright 2017,2018 B.H.J Thate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy 
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights 
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions:
# 
# The above copyright notice and this permission notice don't have to be included.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN 
# THE SOFTWARE.
#
# 18-09-2018 As the creator of this file, I disclaim all rights on this file. 
#
# Bart Thate
# Heerhugowaard
# The Netherlands

""" threading module. """

__version__ = 1

## imports

from obj.base import Obj

import logging
import queue
import time
import threading

## defines

psformat = "%-8s %-60s"
starttime = time.time()

## classes

class Task(threading.Thread):

    def __init__(self, *args, **kwargs):
        super().__init__(None, self.run, "", (), kwargs, daemon=False)
        self.kwargs = kwargs
        self.event = None
        self.queue = queue.Queue()
        self.result = None
        self.starttime = time.time()

    def __iter__(self):
        return self

    def __next__(self):
        for k in dir(self):
            yield k

    def put(self, *args):
        self.queue.put_nowait((args[0], args[1:]))
        return self

    def run(self):
        (func, args) = self.queue.get()
        n = name(func)
        self.starttime = time.time()
        if len(args):
            self.event = args[0]
            if "txt" in dir(self.event):
                n = self.event.get_cmd()
        self.setName(n)
        self.result = func(*args)
        self.elapsed = time.time() - self.starttime
        return self.result

    def join(self, wait=None):
        super().join(wait)
        return self.result

class Worker(Task):

    def run(self):
        self.starttime = time.time()
        self.stopped = False
        while not self.stopped:
            (func, args) = self.queue.get()
            if not func:
                break
            event = None
            n = ""
            if len(args):
                event = args[0]
                try:
                    n = event.txt.split()[0]
                except (IndexError, AttributeError) as ex:
                    pass
            if not n:
                n = name(func)
            self.setName(n)
            try:
                self.result = func(*args)
            except Exception as ex:
                trace = get_exception()
                logging.error(trace)
        self.endtime = time.time()

class Launcher(object):

    def waiter(self, thrs, timeout=None):
        result = []
        for thr in thrs:
            if not thr:
                continue
            try:
                res = thr.join(timeout)
                result.append(res)
            except AttributeError:
                pass
            except KeyboardInterrupt:
                break
        return result

    def launch(self, *args, **kwargs):
        t = Task(**kwargs)
        t.start()
        t.put(*args)
        return t

    def kill(self, thrname=""):
        thrs = []
        for thr in self.running(thrname):
            if "cancel" in dir(thr):
                thr.cancel()
                logging.warn("cancel %s" % str(thr))
            if "exit" in dir(thr):
                thr.exit()
                logging.warn("exit %s" % str(thr))
            if "stop" in dir(thr):
                thr.stop()
                logging.warn("stop %s" % str(thr))
            thrs.append(thr)
        return thrs

    def running(self, tname=""):
        for thr in threading.enumerate():
            name = str(thr)
            if name.startswith("<_"):
                continue
            if tname and tname.upper() not in name.upper():
                continue
            yield thr

## helpers

def name(func):
    try:
        n = "%s.%s" % (func.__self__.__class__.__name__, func.__name__)
    except:
        n = func.__name__
    return n
